package InitialSchedules;

import org.chocosolver.solver.Model;
import org.chocosolver.solver.variables.BoolVar;
import org.chocosolver.solver.variables.IntVar;

public class ModelInitialSchedules {

//	private final int HEURISTIQUE_DEFAUT = 0;
//	private final int HEURISTIQUE_DOMOVERWDEG = 1;
//	private final int HEURISTIQUE_IMPACT_BASED_SEARCH = 2;
//	private final int HEURISTIQUE_ACTIVITY = 3;
//	private final String COHERENCE_BORNES = "BC";
//	private final String COHERENCE_DOMAINES = "AC";
//	private final int RESTART_AUCUN = 0;
//	private final int RESTART_LUBY = 1;
//	private final int RESTART_GEOMETRIQUE = 2;
    private ParametersInitialSchedules myScheduleParameters;

    public int maxPartTimeEmployee;
    public int maxFullTimeEmployee;

    public IntVar workPeriodsSchedulesOfPartTimeEmployees[][];
    private IntVar transposeWorkPeriodsSchedulesOfPartTimeEmployees[][];
    public IntVar workPeriodsSchedulesOfFullTimeEmployees[][];
    private IntVar transposeWorkPeriodsSchedulesOfFullTimeEmployees[][];

    public IntVar workingPeriodsPerPartTimeEmployees[];
    public IntVar partTimeEmployeesPerWorkPeriods[];
    public IntVar workingPeriodsPerFullTimeEmployees[];
    public IntVar fullTimeEmployeesPerWorkPeriods[];
    public IntVar employeesPerWorkPeriods[];

    public Model chocoModelInitialSchedules;
    public IntVar scheduleProfit;

    public ModelInitialSchedules(ParametersInitialSchedules myScheduleParameters) {

        this.myScheduleParameters = myScheduleParameters;

        this.chocoModelInitialSchedules = new Model("Model Initial Schedules");

        this.createEmployeesVariables();
        this.createScheduleVariables();

        this.createModelConstraints();

        this.createModelObjectiveFunction();

    }

    private void createEmployeesVariables() {

        this.maxPartTimeEmployee = (int) Math.ceil((double) myScheduleParameters.totalWorkedPeriodsInSchedule
                / myScheduleParameters.minWorkingPeriodsOfPartTimeEmployeesPerSchedule);
        this.maxFullTimeEmployee = (int) Math.ceil((double) myScheduleParameters.totalWorkedPeriodsInSchedule
                / myScheduleParameters.workingPeriodsOfFullTimeEmployeesPerSchedule);

    }

    private void createScheduleVariables() {

        // Variable pour l'horaire des employes
        this.workPeriodsSchedulesOfPartTimeEmployees = chocoModelInitialSchedules.intVarMatrix(this.maxPartTimeEmployee, this.myScheduleParameters.workPeriodsPerSchedule, 0, 1);
        this.workPeriodsSchedulesOfFullTimeEmployees = chocoModelInitialSchedules.intVarMatrix(this.maxFullTimeEmployee, this.myScheduleParameters.workPeriodsPerSchedule, 0, 1);

        // Variable pour faire le compte du nombre d'heures des employes
        this.workingPeriodsPerPartTimeEmployees = chocoModelInitialSchedules.intVarArray(this.maxPartTimeEmployee, 0, this.myScheduleParameters.workPeriodsPerSchedule, true);
        this.workingPeriodsPerFullTimeEmployees = chocoModelInitialSchedules.intVarArray(this.maxFullTimeEmployee, 0, this.myScheduleParameters.workPeriodsPerSchedule, true);

        // Creer la transpose des horaires pour compter les elements des colonnes
        this.transposeWorkPeriodsSchedulesOfPartTimeEmployees = chocoModelInitialSchedules.intVarMatrix(this.myScheduleParameters.workPeriodsPerSchedule, this.maxPartTimeEmployee, 0, 1);
        for (int workPeriod = 0; workPeriod < this.myScheduleParameters.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
                this.transposeWorkPeriodsSchedulesOfPartTimeEmployees[workPeriod][employee]
                        = this.workPeriodsSchedulesOfPartTimeEmployees[employee][workPeriod];
            }
        }
        this.transposeWorkPeriodsSchedulesOfFullTimeEmployees = chocoModelInitialSchedules.intVarMatrix(this.myScheduleParameters.workPeriodsPerSchedule, this.maxFullTimeEmployee, 0, 1);
        for (int workPeriod = 0; workPeriod < this.myScheduleParameters.workPeriodsPerSchedule; workPeriod++) {
            for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
                this.transposeWorkPeriodsSchedulesOfFullTimeEmployees[workPeriod][employee]
                        = this.workPeriodsSchedulesOfFullTimeEmployees[employee][workPeriod];
            }
        }

        // Variable pour faire le compte du nombre d'employes par periode de travail
        this.partTimeEmployeesPerWorkPeriods = chocoModelInitialSchedules.intVarArray(this.myScheduleParameters.workPeriodsPerSchedule, 0, this.maxPartTimeEmployee, true);
        this.fullTimeEmployeesPerWorkPeriods = chocoModelInitialSchedules.intVarArray(this.myScheduleParameters.workPeriodsPerSchedule, 0, this.maxFullTimeEmployee, true);
        this.employeesPerWorkPeriods = chocoModelInitialSchedules.intVarArray(this.myScheduleParameters.workPeriodsPerSchedule,
                0, this.maxPartTimeEmployee + this.maxFullTimeEmployee, true);

    }

    private void createModelConstraints() {

        // Constrainte tableau pour choisir un type d'horaire dans les horaires possibles
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            chocoModelInitialSchedules.table(this.workPeriodsSchedulesOfFullTimeEmployees[employee],
                    this.myScheduleParameters.enumerationWorkPeriodsSchedulesOfFullTimeEmployees).post();
        }

        // Constraintes pour compter le nombre d'heures travaillees par les differents employes
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            chocoModelInitialSchedules.count(chocoModelInitialSchedules.intVar(1),
                    this.workPeriodsSchedulesOfPartTimeEmployees[employee],
                    this.workingPeriodsPerPartTimeEmployees[employee]).post();
        }
        for (int employee = 0; employee < this.maxFullTimeEmployee; employee++) {
            chocoModelInitialSchedules.count(chocoModelInitialSchedules.intVar(1),
                    this.workPeriodsSchedulesOfFullTimeEmployees[employee],
                    this.workingPeriodsPerFullTimeEmployees[employee]).post();
        }

        // Constraintes pour borner le nombre d'heures travaillees par les employes a temps partiel
        for (int employee = 0; employee < this.maxPartTimeEmployee; employee++) {
            BoolVar isWorkingEmployee = chocoModelInitialSchedules.arithm(
                    this.workingPeriodsPerPartTimeEmployees[employee], ">", 0).reify();
            chocoModelInitialSchedules.ifThen(isWorkingEmployee,
                    chocoModelInitialSchedules.arithm(
                            this.workingPeriodsPerPartTimeEmployees[employee], ">=",
                            this.myScheduleParameters.minWorkingPeriodsOfPartTimeEmployeesPerSchedule));
            chocoModelInitialSchedules.arithm(this.workingPeriodsPerPartTimeEmployees[employee], "<=",
                    this.myScheduleParameters.maxWorkingPeriodsOfPartTimeEmployeesPerSchedule).post();
        }

        // Constraintes pour compter le nombre d'employes par periode de travail et s'assurer qu'il 
        // satisfait la demande en employes
        for (int workPeriod = 0; workPeriod < this.myScheduleParameters.workPeriodsPerSchedule; workPeriod++) {
            chocoModelInitialSchedules.count(chocoModelInitialSchedules.intVar(1),
                    transposeWorkPeriodsSchedulesOfPartTimeEmployees[workPeriod],
                    this.partTimeEmployeesPerWorkPeriods[workPeriod]).post();
            chocoModelInitialSchedules.count(chocoModelInitialSchedules.intVar(1),
                    transposeWorkPeriodsSchedulesOfFullTimeEmployees[workPeriod],
                    this.fullTimeEmployeesPerWorkPeriods[workPeriod]).post();
            chocoModelInitialSchedules.arithm(this.partTimeEmployeesPerWorkPeriods[workPeriod], "+",
                    this.fullTimeEmployeesPerWorkPeriods[workPeriod], "=",
                    this.myScheduleParameters.requiredWorkforce[workPeriod]).post();
            chocoModelInitialSchedules.arithm(this.partTimeEmployeesPerWorkPeriods[workPeriod], "+",
                    this.fullTimeEmployeesPerWorkPeriods[workPeriod], "=",
                    this.employeesPerWorkPeriods[workPeriod]).post();
        }

    }

    private void createModelObjectiveFunction() {

//		this.scheduleProfit = chocoModelInitialSchedules.intVar("TOTAL_COST", 0, 20 * N * p, true);
//		chocoModelInitialSchedules.scalar(perte , coeffs , "=", this.scheduleProfit).post();
    }

}
